package Exporter::Declare::Export::Alias;
use strict;
use warnings;

use base 'Exporter::Declare::Export';

1;

=head1 NAME

Exporter::Declare::Export::Alias - Export class for aliases.

=head1 DESCRIPTION

Export class for aliases. Currently does not expand upon
L<Exporter::Declare::Export> in any way.

=head1 AUTHORS

Chad Granum L<exodist7@gmail.com>

=head1 COPYRIGHT

Copyright (C) 2010 Chad Granum

Exporter-Declare is free software; Standard perl licence.

Exporter-Declare is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the license for more details.
